/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "Heavy_WSTD_FLDR.hpp"

#include <new>

#define Context(_c) static_cast<Heavy_WSTD_FLDR *>(_c)


/*
 * C Functions
 */

extern "C" {
  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new(double sampleRate) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate);
    return Context(ptr);
  }

  HV_EXPORT HeavyContextInterface *hv_WSTD_FLDR_new_with_options(double sampleRate,
      int poolKb, int inQueueKb, int outQueueKb) {
    // allocate aligned memory
    void *ptr = hv_malloc(sizeof(Heavy_WSTD_FLDR));
    // ensure non-null
    if (!ptr) return nullptr;
    // call constructor
    new(ptr) Heavy_WSTD_FLDR(sampleRate, poolKb, inQueueKb, outQueueKb);
    return Context(ptr);
  }

  HV_EXPORT void hv_WSTD_FLDR_free(HeavyContextInterface *instance) {
    // call destructor
    Context(instance)->~Heavy_WSTD_FLDR();
    // free memory
    hv_free(instance);
  }
} // extern "C"







/*
 * Class Functions
 */

Heavy_WSTD_FLDR::Heavy_WSTD_FLDR(double sampleRate, int poolKb, int inQueueKb, int outQueueKb)
    : HeavyContext(sampleRate, poolKb, inQueueKb, outQueueKb) {
  numBytes += sLine_init(&sLine_jGu04y78);
  numBytes += sEnv_init(&sEnv_0X08jzWI, 512, 256);
  numBytes += sEnv_init(&sEnv_TESQ0eg0, 512, 256);
  numBytes += sEnv_init(&sEnv_H2rNzE4K, 512, 256);
  numBytes += sEnv_init(&sEnv_hfdzUN3Y, 512, 256);
  numBytes += sLine_init(&sLine_7xqCqRZ7);
  numBytes += sLine_init(&sLine_mmqd0151);
  numBytes += cVar_init_f(&cVar_eHMCAdVQ, 0.0f);
  numBytes += cVar_init_f(&cVar_VsSb1tLi, 0.0f);
  numBytes += sVarf_init(&sVarf_btryAGMn, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_W1owvaY3, 0.0f);
  numBytes += cIf_init(&cIf_lIXZzAys, false);
  numBytes += cIf_init(&cIf_QKEfrLWY, false);
  numBytes += cBinop_init(&cBinop_hHx2sNdf, 0.0f); // __sub
  numBytes += sVarf_init(&sVarf_VxzSSbfG, 0.0f, 0.0f, false);
  numBytes += cVar_init_f(&cVar_qj4RaNoX, 0.0f);
  numBytes += cIf_init(&cIf_9uVQnNLb, false);
  numBytes += cIf_init(&cIf_IclFBErS, false);
  numBytes += cBinop_init(&cBinop_rWcAXi9V, 0.0f); // __sub
  numBytes += cVar_init_f(&cVar_LxbYdyLM, 1.0f);
  numBytes += sVarf_init(&sVarf_muDQOXAP, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_McwrB8bT, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_1ODBMSKj, 0.0f, 0.0f, false);
  numBytes += sVarf_init(&sVarf_Yf0NaSH7, 0.0f, 0.0f, false);
  
  // schedule a message to trigger all loadbangs via the __hv_init receiver
  scheduleMessageForReceiver(0xCE5CC65B, msg_initWithBang(HV_MESSAGE_ON_STACK(1), 0));
}

Heavy_WSTD_FLDR::~Heavy_WSTD_FLDR() {
  sEnv_free(&sEnv_0X08jzWI);
  sEnv_free(&sEnv_TESQ0eg0);
  sEnv_free(&sEnv_H2rNzE4K);
  sEnv_free(&sEnv_hfdzUN3Y);
}

HvTable *Heavy_WSTD_FLDR::getTableForHash(hv_uint32_t tableHash) {
  return nullptr;
}

void Heavy_WSTD_FLDR::scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) {
  switch (receiverHash) {
    case 0xE42C892E: { // Fldr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_EA6CZ4xQ_sendMessage);
      break;
    }
    case 0xCB23DC60: { // Lmtr
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_tJothsgh_sendMessage);
      break;
    }
    case 0x97D46AC6: { // Mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_Q5NEhsR2_sendMessage);
      break;
    }
    case 0xCF8296B9: { // 1001-mix
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_pNnx42uj_sendMessage);
      break;
    }
    case 0xCE5CC65B: { // __hv_init
      mq_addMessageByTimestamp(&mq, m, 0, &cReceive_SgP2K5bO_sendMessage);
      break;
    }
    default: return;
  }
}

int Heavy_WSTD_FLDR::getParameterInfo(int index, HvParameterInfo *info) {
  if (info != nullptr) {
    switch (index) {
      case 0: {
        info->name = "Fldr";
        info->hash = 0xE42C892E;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 1.0f;
        info->maxVal = 13.37f;
        info->defaultVal = 1.0f;
        break;
      }
      case 1: {
        info->name = "Lmtr";
        info->hash = 0xCB23DC60;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 1.0f;
        info->defaultVal = 1.0f;
        break;
      }
      case 2: {
        info->name = "Mix";
        info->hash = 0x97D46AC6;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 100.0f;
        info->defaultVal = 50.0f;
        break;
      }
      default: {
        info->name = "invalid parameter index";
        info->hash = 0;
        info->type = HvParameterType::HV_PARAM_TYPE_PARAMETER_IN;
        info->minVal = 0.0f;
        info->maxVal = 0.0f;
        info->defaultVal = 0.0f;
        break;
      }
    }
  }
  return 3;
}



/*
 * Send Function Implementations
 */


void Heavy_WSTD_FLDR::cBinop_wXonC9ex_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MAX, 0.0f, 0, m, &cBinop_fGm6PrP0_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_fGm6PrP0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 2.0f, 0, m, &cBinop_EBFAiFw5_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_EBFAiFw5_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 1.0f, 0, m, &cBinop_ZgGSudm2_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_ZgGSudm2_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_IkKGmGBt_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_U2ns78fH_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_IkKGmGBt_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_ADD, 1.0f, 0, m, &cBinop_Eoz3XNQw_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_U2ns78fH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_wVpy5rLM_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_Eoz3XNQw_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_1rFguaNc_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_wVpy5rLM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_rdZasiNl_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_rdZasiNl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MULTIPLY, 0.5f, 0, m, &cBinop_YSSrzkkR_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_1rFguaNc_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_Ql8VUQom_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_YSSrzkkR_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_WPaY2jCz_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_eHMCAdVQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cSend_Hn0IWCWp_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_qnzo755X_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_V7Ccq3r0_sendMessage(_c, 0, m);
  cMsg_BCTlu6UL_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_VsSb1tLi_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_uQdP7CVQ_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cVar_W1owvaY3_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_hHx2sNdf, HV_BINOP_SUBTRACT, 0, m, &cBinop_hHx2sNdf_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_lIXZzAys_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_btryAGMn, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_QKEfrLWY_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_btryAGMn, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_0X08jzWI_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_W1owvaY3, 0, m, &cVar_W1owvaY3_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_TESQ0eg0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_IdUnvFaZ_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_viW3ZRdF_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_oTNGJsxl_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_hHx2sNdf_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_eQmwr0lr_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_eQmwr0lr_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_W3Lyd2r0_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_W3Lyd2r0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_Rsj1OhUH_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_Khq4s7cJ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_Lu4elQXq_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_nv3m3iW8_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_Lu4elQXq_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_lIXZzAys, 1, m, &cIf_lIXZzAys_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_nv3m3iW8_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_QlgvF7HF_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_rxuu5fNb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_QKEfrLWY, 1, m, &cIf_QKEfrLWY_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_QlgvF7HF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_lIXZzAys, 0, m, &cIf_lIXZzAys_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_Rsj1OhUH_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_QKEfrLWY, 0, m, &cIf_QKEfrLWY_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_IdUnvFaZ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_oTNGJsxl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_W1owvaY3, 0, m, &cVar_W1owvaY3_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_viW3ZRdF_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_hHx2sNdf, HV_BINOP_SUBTRACT, 1, m, &cBinop_hHx2sNdf_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_qj4RaNoX_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_rWcAXi9V, HV_BINOP_SUBTRACT, 0, m, &cBinop_rWcAXi9V_sendMessage);
}

void Heavy_WSTD_FLDR::cIf_9uVQnNLb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_VxzSSbfG, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::cIf_IclFBErS_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  switch (letIn) {
    case 0: {
      break;
    }
    case 1: {
      sVarf_onMessage(_c, &Context(_c)->sVarf_VxzSSbfG, m);
      break;
    }
    default: return;
  }
}

void Heavy_WSTD_FLDR::sEnv_H2rNzE4K_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_qj4RaNoX, 0, m, &cVar_qj4RaNoX_sendMessage);
}

void Heavy_WSTD_FLDR::sEnv_hfdzUN3Y_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_mOdhrEQd_sendMessage);
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_7LVJUYdg_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_HXe6mhPz_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_rWcAXi9V_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_FNdNM8Pk_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cMsg_FNdNM8Pk_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  msg_setElementToFrom(m, 1, n, 0);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_SUBTRACT, 0.0f, 0, m, &cBinop_uUiJy9ez_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_uUiJy9ez_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 100.0f, 0, m, &cBinop_qsaObh7Q_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_cXXRTCDD_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_FLOAT, 0, m, &cCast_s1I4MVeb_sendMessage);
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_hPmWV6e9_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_hPmWV6e9_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cMsg_uPXwlpvm_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cCast_s1I4MVeb_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_9uVQnNLb, 1, m, &cIf_9uVQnNLb_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_HUkUyg8R_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_IclFBErS, 1, m, &cIf_IclFBErS_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_uPXwlpvm_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 100.0f);
  cIf_onMessage(_c, &Context(_c)->cIf_9uVQnNLb, 0, m, &cIf_9uVQnNLb_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_qsaObh7Q_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cIf_onMessage(_c, &Context(_c)->cIf_IclFBErS, 0, m, &cIf_IclFBErS_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_7LVJUYdg_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_onMessage(_c, &Context(_c)->cBinop_rWcAXi9V, HV_BINOP_SUBTRACT, 1, m, &cBinop_rWcAXi9V_sendMessage);
}

void Heavy_WSTD_FLDR::cCast_mOdhrEQd_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
}

void Heavy_WSTD_FLDR::cCast_HXe6mhPz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_qj4RaNoX, 0, m, &cVar_qj4RaNoX_sendMessage);
}

void Heavy_WSTD_FLDR::cVar_LxbYdyLM_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_RExD9f4T_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_Khq4s7cJ_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_rxuu5fNb_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 0.0f, 0, m, &cBinop_cXXRTCDD_sendMessage);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_EQ, 1.0f, 0, m, &cBinop_HUkUyg8R_sendMessage);
}

void Heavy_WSTD_FLDR::cBinop_RExD9f4T_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_LxbYdyLM, 1, m, &cVar_LxbYdyLM_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_Ql8VUQom_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_mmqd0151, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_WPaY2jCz_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 100.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_7xqCqRZ7, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cSend_Hn0IWCWp_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cReceive_pNnx42uj_sendMessage(_c, 0, m);
}

void Heavy_WSTD_FLDR::cBinop_eYcHx6Fl_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_eHMCAdVQ, 0, m, &cVar_eHMCAdVQ_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_BCTlu6UL_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 50.0f);
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_eYcHx6Fl_sendMessage);
}

void Heavy_WSTD_FLDR::cMsg_uQdP7CVQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(2);
  msg_init(m, 2, msg_getTimestamp(n));
  msg_setElementToFrom(m, 0, n, 0);
  msg_setFloat(m, 1, 20.0f);
  sLine_onMessage(_c, &Context(_c)->sLine_jGu04y78, 0, m, NULL);
}

void Heavy_WSTD_FLDR::cMsg_V7Ccq3r0_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *const n) {
  HvMessage *m = nullptr;
  m = HV_MESSAGE_ON_STACK(1);
  msg_init(m, 1, msg_getTimestamp(n));
  msg_setFloat(m, 0, 1.0f);
  cVar_onMessage(_c, &Context(_c)->cVar_VsSb1tLi, 0, m, &cVar_VsSb1tLi_sendMessage);
  cVar_onMessage(_c, &Context(_c)->cVar_LxbYdyLM, 0, m, &cVar_LxbYdyLM_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_pNnx42uj_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_MIN, 1.0f, 0, m, &cBinop_wXonC9ex_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_Q5NEhsR2_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cBinop_k_onMessage(_c, NULL, HV_BINOP_DIVIDE, 100.0f, 0, m, &cBinop_eYcHx6Fl_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_SgP2K5bO_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cCast_onMessage(_c, HV_CAST_BANG, 0, m, &cCast_qnzo755X_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_tJothsgh_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_LxbYdyLM, 0, m, &cVar_LxbYdyLM_sendMessage);
}

void Heavy_WSTD_FLDR::cReceive_EA6CZ4xQ_sendMessage(HeavyContextInterface *_c, int letIn, const HvMessage *m) {
  cVar_onMessage(_c, &Context(_c)->cVar_VsSb1tLi, 0, m, &cVar_VsSb1tLi_sendMessage);
}




/*
 * Context Process Implementation
 */

int Heavy_WSTD_FLDR::process(float **inputBuffers, float **outputBuffers, int n) {
  while (hLp_hasData(&inQueue)) {
    hv_uint32_t numBytes = 0;
    ReceiverMessagePair *p = reinterpret_cast<ReceiverMessagePair *>(hLp_getReadBuffer(&inQueue, &numBytes));
    hv_assert(numBytes >= sizeof(ReceiverMessagePair));
    scheduleMessageForReceiver(p->receiverHash, &p->msg);
    hLp_consume(&inQueue);
  }

  sendBangToReceiver(0xDD21C0EB); // send to __hv_bang~ on next cycle
  const int n4 = n & ~HV_N_SIMD_MASK; // ensure that the block size is a multiple of HV_N_SIMD

  // temporary signal vars
  hv_bufferf_t Bf0, Bf1, Bf2, Bf3;

  // input and output vars
  hv_bufferf_t O0, O1;
  hv_bufferf_t I0, I1;

  // declare and init the zero buffer
  hv_bufferf_t ZERO; __hv_zero_f(VOf(ZERO));

  hv_uint32_t nextBlock = blockStartTimestamp;
  for (int n = 0; n < n4; n += HV_N_SIMD) {

    // process all of the messages for this block
    nextBlock += HV_N_SIMD;
    while (mq_hasMessageBefore(&mq, nextBlock)) {
      MessageNode *const node = mq_peek(&mq);
      node->sendMessage(this, node->let, node->m);
      mq_pop(&mq);
    }

    // load input buffers
    __hv_load_f(inputBuffers[0]+n, VOf(I0));
    __hv_load_f(inputBuffers[1]+n, VOf(I1));

    // zero output buffers
    __hv_zero_f(VOf(O0));
    __hv_zero_f(VOf(O1));

    // process all signal functions
    __hv_varwrite_f(&sVarf_muDQOXAP, VIf(I0));
    __hv_line_f(&sLine_jGu04y78, VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I0), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_floor_f(VIf(Bf2), VOf(Bf1));
    __hv_sub_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_abs_f(VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf2), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_0X08jzWI, VIf(Bf3), &sEnv_0X08jzWI_sendMessage);
    sEnv_process(this, &sEnv_TESQ0eg0, VIf(I0), &sEnv_TESQ0eg0_sendMessage);
    __hv_varread_f(&sVarf_btryAGMn, VOf(Bf1));
    __hv_var_k_f(VOf(Bf2), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf2), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_McwrB8bT, VIf(Bf1));
    __hv_varwrite_f(&sVarf_1ODBMSKj, VIf(I1));
    __hv_var_k_f(VOf(Bf1), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fma_f(VIf(I1), VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_div_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_floor_f(VIf(Bf0), VOf(Bf1));
    __hv_sub_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f, 0.5f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_abs_f(VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f, 4.0f);
    __hv_var_k_f(VOf(Bf3), 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f, 1.0f);
    __hv_fms_f(VIf(Bf0), VIf(Bf1), VIf(Bf3), VOf(Bf3));
    __hv_neg_f(VIf(Bf3), VOf(Bf3));
    sEnv_process(this, &sEnv_H2rNzE4K, VIf(Bf3), &sEnv_H2rNzE4K_sendMessage);
    sEnv_process(this, &sEnv_hfdzUN3Y, VIf(I1), &sEnv_hfdzUN3Y_sendMessage);
    __hv_varread_f(&sVarf_VxzSSbfG, VOf(Bf1));
    __hv_var_k_f(VOf(Bf0), 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f, 100.0f);
    __hv_sub_f(VIf(Bf1), VIf(Bf0), VOf(Bf0));
    __hv_var_k_f(VOf(Bf1), 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f, 0.115129254649702f);
    __hv_mul_f(VIf(Bf0), VIf(Bf1), VOf(Bf1));
    __hv_exp_f(VIf(Bf1), VOf(Bf1));
    __hv_mul_f(VIf(Bf3), VIf(Bf1), VOf(Bf1));
    __hv_varwrite_f(&sVarf_Yf0NaSH7, VIf(Bf1));
    __hv_varread_f(&sVarf_muDQOXAP, VOf(Bf1));
    __hv_line_f(&sLine_7xqCqRZ7, VOf(Bf3));
    __hv_varread_f(&sVarf_McwrB8bT, VOf(Bf0));
    __hv_line_f(&sLine_mmqd0151, VOf(Bf2));
    __hv_mul_f(VIf(Bf0), VIf(Bf2), VOf(Bf0));
    __hv_fma_f(VIf(Bf1), VIf(Bf3), VIf(Bf0), VOf(Bf0));
    __hv_add_f(VIf(Bf0), VIf(O0), VOf(O0));
    __hv_varread_f(&sVarf_1ODBMSKj, VOf(Bf0));
    __hv_varread_f(&sVarf_Yf0NaSH7, VOf(Bf1));
    __hv_mul_f(VIf(Bf1), VIf(Bf2), VOf(Bf2));
    __hv_fma_f(VIf(Bf0), VIf(Bf3), VIf(Bf2), VOf(Bf2));
    __hv_add_f(VIf(Bf2), VIf(O1), VOf(O1));

    // save output vars to output buffer
    __hv_store_f(outputBuffers[0]+n, VIf(O0));
    __hv_store_f(outputBuffers[1]+n, VIf(O1));
  }

  blockStartTimestamp = nextBlock;

  return n4; // return the number of frames processed

}

int Heavy_WSTD_FLDR::processInline(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(!(n4 & HV_N_SIMD_MASK)); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s)
  float **const bIn = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bIn[0] = inputBuffers+(0*n4);
  bIn[1] = inputBuffers+(1*n4);

  // define the heavy output buffer for 2 channel(s)
  float **const bOut = reinterpret_cast<float **>(hv_alloca(2*sizeof(float *)));
  bOut[0] = outputBuffers+(0*n4);
  bOut[1] = outputBuffers+(1*n4);

  int n = process(bIn, bOut, n4);
  return n;
}

int Heavy_WSTD_FLDR::processInlineInterleaved(float *inputBuffers, float *outputBuffers, int n4) {
  hv_assert(n4 & ~HV_N_SIMD_MASK); // ensure that n4 is a multiple of HV_N_SIMD

  // define the heavy input buffer for 2 channel(s), uninterleave
  float *const bIn = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));
  #if HV_SIMD_SSE || HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 a = _mm_load_ps(inputBuffers+i);                // LRLR
    __m128 b = _mm_load_ps(inputBuffers+4+i);              // LRLR
    __m128 x = _mm_shuffle_ps(a, b, _MM_SHUFFLE(2,0,2,0)); // LLLL
    __m128 y = _mm_shuffle_ps(a, b, _MM_SHUFFLE(3,1,3,1)); // RRRR
    _mm_store_ps(bIn+j, x);
    _mm_store_ps(bIn+n4+j, y);
  }
  #elif HV_SIMD_NEON
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4x2_t a = vld2q_f32(inputBuffers+i); // load and uninterleave
    vst1q_f32(bIn+j, a.val[0]);
    vst1q_f32(bIn+n4+j, a.val[1]);
  }
  #else // HV_SIMD_NONE
  for (int j = 0; j < n4; ++j) {
    bIn[0*n4+j] = inputBuffers[0+2*j];
    bIn[1*n4+j] = inputBuffers[1+2*j];
  }
  #endif

  // define the heavy output buffer for 2 channel(s)
  float *const bOut = reinterpret_cast<float *>(hv_alloca(2*n4*sizeof(float)));

  int n = processInline(bIn, bOut, n4);

  // interleave the heavy output into the output buffer
  #if HV_SIMD_AVX
  for (int i = 0, j = 0; j < n4; j += 8, i += 16) {
    __m256 x = _mm256_load_ps(bOut+j);    // LLLLLLLL
    __m256 y = _mm256_load_ps(bOut+n4+j); // RRRRRRRR
    __m256 a = _mm256_unpacklo_ps(x, y);  // LRLRLRLR
    __m256 b = _mm256_unpackhi_ps(x, y);  // LRLRLRLR
    _mm256_store_ps(outputBuffers+i, a);
    _mm256_store_ps(outputBuffers+8+i, b);
  }
  #elif HV_SIMD_SSE
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    __m128 x = _mm_load_ps(bOut+j);    // LLLL
    __m128 y = _mm_load_ps(bOut+n4+j); // RRRR
    __m128 a = _mm_unpacklo_ps(x, y);  // LRLR
    __m128 b = _mm_unpackhi_ps(x, y);  // LRLR
    _mm_store_ps(outputBuffers+i, a);
    _mm_store_ps(outputBuffers+4+i, b);
  }
  #elif HV_SIMD_NEON
  // https://community.arm.com/groups/processors/blog/2012/03/13/coding-for-neon--part-5-rearranging-vectors
  for (int i = 0, j = 0; j < n4; j += 4, i += 8) {
    float32x4_t x = vld1q_f32(bOut+j);
    float32x4_t y = vld1q_f32(bOut+n4+j);
    float32x4x2_t z = {x, y};
    vst2q_f32(outputBuffers+i, z); // interleave and store
  }
  #else // HV_SIMD_NONE
  for (int i = 0; i < 2; ++i) {
    for (int j = 0; j < n4; ++j) {
      outputBuffers[i+2*j] = bOut[i*n4+j];
    }
  }
  #endif

  return n;
}
