/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalLine.h"
#include "HvSignalEnvelope.h"
#include "HvSignalVar.h"
#include "HvControlVar.h"
#include "HvControlCast.h"
#include "HvControlIf.h"
#include "HvControlBinop.h"
#include "HvMath.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_wXonC9ex_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fGm6PrP0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_EBFAiFw5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZgGSudm2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IkKGmGBt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_U2ns78fH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Eoz3XNQw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_wVpy5rLM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rdZasiNl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1rFguaNc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_YSSrzkkR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_eHMCAdVQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qnzo755X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_VsSb1tLi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_W1owvaY3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_lIXZzAys_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_QKEfrLWY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_0X08jzWI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_TESQ0eg0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hHx2sNdf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_eQmwr0lr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_W3Lyd2r0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Khq4s7cJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Lu4elQXq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_nv3m3iW8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rxuu5fNb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_QlgvF7HF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Rsj1OhUH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IdUnvFaZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_oTNGJsxl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_viW3ZRdF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_qj4RaNoX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_9uVQnNLb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_IclFBErS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_H2rNzE4K_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_hfdzUN3Y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_rWcAXi9V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_FNdNM8Pk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_uUiJy9ez_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_cXXRTCDD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hPmWV6e9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_s1I4MVeb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HUkUyg8R_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uPXwlpvm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qsaObh7Q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_7LVJUYdg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mOdhrEQd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_HXe6mhPz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_LxbYdyLM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RExD9f4T_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Ql8VUQom_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_WPaY2jCz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_Hn0IWCWp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_eYcHx6Fl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BCTlu6UL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uQdP7CVQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_V7Ccq3r0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_pNnx42uj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Q5NEhsR2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_SgP2K5bO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_tJothsgh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_EA6CZ4xQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_jGu04y78;
  SignalEnvelope sEnv_0X08jzWI;
  SignalEnvelope sEnv_TESQ0eg0;
  SignalEnvelope sEnv_H2rNzE4K;
  SignalEnvelope sEnv_hfdzUN3Y;
  SignalLine sLine_7xqCqRZ7;
  SignalLine sLine_mmqd0151;
  ControlBinop cBinop_wXonC9ex;
  ControlBinop cBinop_fGm6PrP0;
  ControlBinop cBinop_EBFAiFw5;
  ControlBinop cBinop_ZgGSudm2;
  ControlBinop cBinop_Eoz3XNQw;
  ControlBinop cBinop_rdZasiNl;
  ControlBinop cBinop_1rFguaNc;
  ControlBinop cBinop_YSSrzkkR;
  ControlVar cVar_eHMCAdVQ;
  ControlVar cVar_VsSb1tLi;
  SignalVarf sVarf_btryAGMn;
  ControlVar cVar_W1owvaY3;
  ControlIf cIf_lIXZzAys;
  ControlIf cIf_QKEfrLWY;
  ControlBinop cBinop_hHx2sNdf;
  ControlBinop cBinop_W3Lyd2r0;
  ControlBinop cBinop_Khq4s7cJ;
  ControlBinop cBinop_rxuu5fNb;
  ControlBinop cBinop_Rsj1OhUH;
  SignalVarf sVarf_VxzSSbfG;
  ControlVar cVar_qj4RaNoX;
  ControlIf cIf_9uVQnNLb;
  ControlIf cIf_IclFBErS;
  ControlBinop cBinop_rWcAXi9V;
  ControlBinop cBinop_uUiJy9ez;
  ControlBinop cBinop_cXXRTCDD;
  ControlBinop cBinop_HUkUyg8R;
  ControlBinop cBinop_qsaObh7Q;
  ControlVar cVar_LxbYdyLM;
  ControlBinop cBinop_RExD9f4T;
  SignalVarf sVarf_muDQOXAP;
  SignalVarf sVarf_McwrB8bT;
  SignalVarf sVarf_1ODBMSKj;
  SignalVarf sVarf_Yf0NaSH7;
  ControlBinop cBinop_eYcHx6Fl;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
